diff --git a/lists.lisp b/lists.lisp
index 00b42fa..3b1e742 100644
--- a/lists.lisp
+++ b/lists.lisp
@@ -341,15 +341,20 @@ Example:
                             one)))))
     (%map-product (ensure-function function) (cons list more-lists))))
 
-(defun flatten (tree)
-  "Traverses the tree in order, collecting non-null leaves into a list."
+(defun flatten (tree &key depth)
+  "Traverses the tree in order, collecting non-null leaves into a
+list. If DEPTH is NIL, the entire tree is flattened, otherwise it is
+flattened by only DEPTH levels."
+  (declare (type depth (or null unsigned-byte)))
   (let (list)
-    (labels ((traverse (subtree)
+    (labels ((traverse (subtree current-depth)
                (when subtree
-                 (if (consp subtree)
+                 (if (and (consp subtree)
+                          (or (null depth)
+                              (>= depth current-depth)))
                      (progn
-                       (traverse (car subtree))
-                       (traverse (cdr subtree)))
+                       (traverse (car subtree) (1+ current-depth))
+                       (traverse (cdr subtree) current-depth))
                      (push subtree list)))))
-      (traverse tree))
+      (traverse tree 0))
     (nreverse list)))
diff --git a/tests.lisp b/tests.lisp
index 603157b..8de59d4 100644
--- a/tests.lisp
+++ b/tests.lisp
@@ -850,6 +850,36 @@
     (flatten '((1) 2 (((3 4))) ((((5)) 6)) 7))
   (1 2 3 4 5 6 7))
 
+(deftest flatten.2
+    (flatten '(0 (1) ((2 (3))) (((3 (4)))) ((((4)))))
+             :depth 0)
+  (0 (1) ((2 (3))) (((3 (4)))) ((((4))))))
+
+(deftest flatten.
+    (flatten '(0 (1) ((2 (3))) (((3 (4)))) ((((4)))))
+             :depth 1)
+  (0 1 (2 (3)) ((3 (4))) (((4)))))
+
+(deftest flatten.4
+    (flatten '(0 (1) ((2 (3))) (((3 (4)))) ((((4)))))
+             :depth 2)
+  (0 1 2 (3) (3 (4)) ((4))))
+
+(deftest flatten.5
+    (flatten '(0 (1) ((2 (3))) (((3 (4)))) ((((4)))))
+             :depth 3)
+  (0 1 2 3 3 (4) (4)))
+
+(deftest flatten.6
+    (flatten '(0 (1) ((2 (3))) (((3 (4)))) ((((4)))))
+             :depth 4)
+  (0 1 2 3 3 4 4))
+
+(deftest flatten.7
+    (flatten '(0 (1) ((2 (3))) (((3 (4)))) ((((4)))))
+             :depth 5)
+  (0 1 2 3 3 4 4))
+
 (deftest remove-from-plist.1
     (let ((orig '(a 1 b 2 c 3 d 4)))
       (list (remove-from-plist orig 'a 'c)
